package de.intevation.flys.artifacts.model;

import gnu.trove.TDoubleArrayList;

import org.apache.log4j.Logger;


/**
 * This class represents a pool of data triples that consists of 'W', 'Q' and
 * 'KM' data.
 *
 * @author <a href="mailto:ingo.weinzierl@intevation.de">Ingo Weinzierl</a>
 */
public class WQKms extends NamedObject {

    private static Logger logger = Logger.getLogger(WQKms.class);


    /** The array that contains the 'W' values.*/
    protected TDoubleArrayList w;

    /** The array that contains the 'Q' values.*/
    protected TDoubleArrayList q;

    /** The array that contains the 'KMs' values.*/
    protected TDoubleArrayList kms;


    public WQKms() {
        this("");
    }


    public WQKms(String name) {
        super(name);

        this.w   = new TDoubleArrayList();
        this.q   = new TDoubleArrayList();
        this.kms = new TDoubleArrayList();
    }


    public WQKms(int capacity) {
        this(capacity, "");
    }


    public WQKms(int capacity, String name) {
        super(name);

        this.w   = new TDoubleArrayList(capacity);
        this.q   = new TDoubleArrayList(capacity);
        this.kms = new TDoubleArrayList(capacity);
    }

    public static void removeNaNs(TDoubleArrayList [] arrays) {

        int dest = 0;

        int A = arrays.length;
        int N = arrays[0].size();

        OUTER: for (int i = 0; i < N; ++i) {
            for (int j = 0; j < A; ++j) {
                TDoubleArrayList a = arrays[j];
                double v = a.getQuick(i);
                if (Double.isNaN(v)) {
                    continue OUTER;
                }
                a.setQuick(dest, v);
            }
            ++dest;
        }

        if (dest < N) {
            for (int i = 0; i < A; ++i) {
                arrays[i].remove(dest, N-dest);
            }
        }
    }

    public void removeNaNs() {
        removeNaNs(new TDoubleArrayList [] { w, q, kms });
    }


    public WQKms(double[] kms, double[] qs, double[] ws) {
        this(kms, qs, ws, "");
    }


    public WQKms(double[] kms, double[] qs, double[] ws, String name) {
        super(name);

        this.w   = new TDoubleArrayList(ws);
        this.q   = new TDoubleArrayList(qs);
        this.kms = new TDoubleArrayList(kms);
    }


    /**
     * Adds a new row to this data pool.
     *
     * @param w a W.
     * @param q a Q.
     * @param kms a Kms.
     */
    public void add(double w, double q, double kms) {
        this.w.add(w);
        this.q.add(q);
        this.kms.add(kms);
    }

    /**
     * Returns the number of triples stored in this data pool.
     *
     * @return the number of triples stored in this data pool.
     */
    public int size() {
        return kms.size();
    }

    /**
     * This method returns a triple of W, Q and Kms in a single 3dim array.
     *
     * @param idx The position of the triple.
     * @param dst destination array
     *
     * @return a triple of [W, Q, Kms] in dst.
     */
    public double[] get(int idx, double [] dst) {
        dst[0] = w  .getQuick(idx);
        dst[1] = q  .getQuick(idx);
        dst[2] = kms.getQuick(idx);
        return dst;
    }


    public double getKms(int idx) {
        return kms.getQuick(idx);
    }

    public double getW(int idx) {
        return w.getQuick(idx);
    }

    public double getQ(int idx) {
        return q.getQuick(idx);
    }

    public double[] getKms() {
        return kms.toNativeArray();
    }


    public double[] getWs() {
        return w.toNativeArray();
    }


    public double[] getQs() {
        return q.toNativeArray();
    }


    /**
     * Returns a string that consist of the first and last kilometer.
     *
     * @return a string that consist of the first and last kilometer.
     */
    public String toString() {
        double from = getKms(0);
        double to   = getKms(size()-1);
        return from + " - " + to;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
